//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * The following disclaimer summarizes all the specific disclaimers *
// * of contributors to this software. The specific disclaimers,which *
// * govern, are listed with their locations in:                      *
// *   http://cern.ch/geant4/license                                  *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * This  code  implementation is the  intellectual property  of the *
// * GEANT4 collaboration.                                            *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
//
// $Id: A01EventAction.cc,v 1.5 2003/08/14 23:27:26 duns Exp $
// --------------------------------------------------------------
//

#include "A01EventAction.hh"
#include "A01EventActionMessenger.hh"

#include "G4Event.hh"
#include "G4EventManager.hh"
#include "G4HCofThisEvent.hh"
#include "G4VHitsCollection.hh"
#include "G4TrajectoryContainer.hh"
#include "G4Trajectory.hh"
#include "G4VVisManager.hh"
#include "G4SDManager.hh"
#include "G4UImanager.hh"
#include "G4ios.hh"

#include "A01HodoscopeHit.hh"
#include "A01DriftChamberHit.hh"

//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
//#include "A01EmCalorimeterHit.hh"
//#include "A01HadCalorimeterHit.hh"
//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

A01EventAction::A01EventAction()
{
  G4String colName;
  G4SDManager* SDman = G4SDManager::GetSDMpointer();
  HHC1ID = SDman->GetCollectionID(colName="hodoscope1/hodoscopeColl");
  DHC1ID = SDman->GetCollectionID(colName="chamber1/driftChamberColl");
//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
//  HHC2ID = SDman->GetCollectionID(colName="hodoscope2/hodoscopeColl");
//  DHC2ID = SDman->GetCollectionID(colName="chamber2/driftChamberColl");
//  ECHCID = SDman->GetCollectionID(colName="EMcalorimeter/EMcalorimeterColl");
//  HCHCID = SDman->GetCollectionID(colName="HadCalorimeter/HadCalorimeterColl");
//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  verboseLevel = 1;
  messenger = new A01EventActionMessenger(this);

}

A01EventAction::~A01EventAction()
{
  delete messenger;
}

void A01EventAction::BeginOfEventAction(const G4Event*)
{
}

void A01EventAction::EndOfEventAction(const G4Event* evt)
{
  G4HCofThisEvent * HCE = evt->GetHCofThisEvent();
  A01HodoscopeHitsCollection* HHC1 = 0;
  A01DriftChamberHitsCollection* DHC1 = 0;
//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
//  A01HodoscopeHitsCollection* HHC2 = 0;
//  A01DriftChamberHitsCollection* DHC2 = 0;
//  A01EmCalorimeterHitsCollection* ECHC = 0;
//  A01HadCalorimeterHitsCollection* HCHC = 0;
//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  if(HCE)
  {
    HHC1 = (A01HodoscopeHitsCollection*)(HCE->GetHC(HHC1ID));
    DHC1 = (A01DriftChamberHitsCollection*)(HCE->GetHC(DHC1ID));
//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
//    HHC2 = (A01HodoscopeHitsCollection*)(HCE->GetHC(HHC2ID));
//    DHC2 = (A01DriftChamberHitsCollection*)(HCE->GetHC(DHC2ID));
//    ECHC = (A01EmCalorimeterHitsCollection*)(HCE->GetHC(ECHCID));
//    HCHC = (A01HadCalorimeterHitsCollection*)(HCE->GetHC(HCHCID));
//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  }

  // Diagnostics

  if (verboseLevel==0 || evt->GetEventID() % verboseLevel != 0) return;

  G4PrimaryParticle* primary = evt->GetPrimaryVertex(0)->GetPrimary(0);
  G4cout << G4endl
         << ">>> Event " << evt->GetEventID() << " >>> Simulation truth : "
         << primary->GetG4code()->GetParticleName()
         << " " << primary->GetMomentum() << G4endl;

  if(HHC1)
  {
    int n_hit = HHC1->entries();
    G4cout << "Hodoscope 1 has " << n_hit << " hits." << G4endl;
    for(int i1=0;i1<n_hit;i1++)
    {
      A01HodoscopeHit* aHit = (*HHC1)[i1];
      aHit->Print();
    }
  }
//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
//  if(HHC2)
//  {
//    int n_hit = HHC2->entries();
//    G4cout << "Hodoscope 2 has " << n_hit << " hits." << G4endl;
//    for(int i1=0;i1<n_hit;i1++)
//    {
//      A01HodoscopeHit* aHit = (*HHC2)[i1];
//      aHit->Print();
//    }
//  }
//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  if(DHC1)
  {
    int n_hit = DHC1->entries();
    G4cout << "Drift Chamber 1 has " << n_hit << " hits." << G4endl;
    for(int i2=0;i2<5;i2++)
    {
      for(int i1=0;i1<n_hit;i1++)
      {
        A01DriftChamberHit* aHit = (*DHC1)[i1];
        if(aHit->GetLayerID()==i2) aHit->Print();
      }
    }
  }
//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
//  if(DHC2)
//  {
//    int n_hit = DHC2->entries();
//    G4cout << "Drift Chamber 2 has " << n_hit << " hits." << G4endl;
//    for(int i2=0;i2<5;i2++)
//    {
//      for(int i1=0;i1<n_hit;i1++)
//      {
//        A01DriftChamberHit* aHit = (*DHC2)[i1];
//        if(aHit->GetLayerID()==i2) aHit->Print();
//      }
//    }
//  }
//  if(ECHC)
//  {
//    int iHit = 0;
//    double totalE = 0.;
//    for(int i1=0;i1<80;i1++)
//    {
//      A01EmCalorimeterHit* aHit = (*ECHC)[i1];
//      double eDep = aHit->GetEdep();
//      if(eDep>0.)
//      {
//        iHit++;
//        totalE += eDep;
//      }
//    }
//    G4cout << "EM Calorimeter has " << iHit << " hits. Total Edep is "
//           << totalE/MeV << " (MeV)" << G4endl;
//  }
//  if(HCHC)
//  {
//    int iHit = 0;
//    double totalE = 0.;
//    for(int i1=0;i1<20;i1++)
//    {
//      A01HadCalorimeterHit* aHit = (*HCHC)[i1];
//      double eDep = aHit->GetEdep();
//      if(eDep>0.)
//      {
//        iHit++;
//        totalE += eDep;
//      }
//    }
//    G4cout << "Hadron Calorimeter has " << iHit << " hits. Total Edep is "
//           << totalE/MeV << " (MeV)" << G4endl;
//  }
//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
}


